{% extends "emails/base.html" %}

{% block content %}
<p>
    You're receiving this email because you requested a password reset for your user account.
</p>
<p>
    Please go to the following page and choose a new password:<br>
    <a href="{{ confirm_reset_url }}">{{ confirm_reset_url }}</a><br>
</p>
{% endblock %}
